@@ -17,18 +17,20 @@
 
 # Authors: Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from functools import partial
 
 import numpy as np
-from scipy import linalg
 import itertools
 
+from .event import find_events
 from .io.base import BaseRaw
-from .io.meas_info import _simplify_info
+from .io.kit.constants import KIT
+from .io.kit.kit import RawKIT as _RawKIT
+from .io.meas_info import _simplify_info, Info
 from .io.pick import (pick_types, pick_channels, pick_channels_regexp,
-                      pick_info)
+                      pick_info, _picks_to_idx)
 from .io.proj import Projection, setup_proj
 from .io.constants import FIFF
 from .io.ctf.trans import _make_ctf_coord_trans_set
@@ -37,13 +39,14 @@ from .forward import (_magnetic_dipole_field_vec, _create_meg_coils,
 from .cov import make_ad_hoc_cov, compute_whitener
 from .dipole import _make_guesses
 from .fixes import jit
-from .preprocessing.maxwell import (_sss_basis, _prep_mf_coils, _get_mf_picks,
-                                    _regularize_out)
+from .preprocessing.maxwell import (_sss_basis, _prep_mf_coils,
+                                    _regularize_out, _get_mf_picks_fix_mags)
 from .transforms import (apply_trans, invert_transform, _angle_between_quats,
                          quat_to_rot, rot_to_quat, _fit_matched_points,
-                         _quat_to_affine)
+                         _quat_to_affine, als_ras_trans)
 from .utils import (verbose, logger, use_log_level, _check_fname, warn,
-                    _validate_type, ProgressBar, _check_option)
+                    _validate_type, ProgressBar, _check_option, _pl,
+                    _on_missing)
 
 # Eventually we should add:
 #   hpicons
@@ -212,18 +215,123 @@ def extract_chpi_locs_ctf(raw, verbose=None):
     return dict(rrs=rrs, gofs=gofs, times=times, moments=moments)
 
 
+@verbose
+def extract_chpi_locs_kit(raw, stim_channel='MISC 064', *, verbose=None):
+    """Extract cHPI locations from KIT data.
+
+    Parameters
+    ----------
+    raw : instance of RawKIT
+        Raw data with KIT cHPI information.
+    stim_channel : str
+        The stimulus channel that encodes HPI measurement intervals.
+    %(verbose)s
+
+    Returns
+    -------
+    %(chpi_locs)s
+
+    Notes
+    -----
+    .. versionadded:: 0.23
+    """
+    _validate_type(raw, (_RawKIT,), 'raw')
+    stim_chs = [
+        raw.info['ch_names'][pick] for pick in pick_types(
+            raw.info, stim=True, misc=True, ref_meg=False)]
+    _validate_type(stim_channel, str, 'stim_channel')
+    _check_option('stim_channel', stim_channel, stim_chs)
+    idx = raw.ch_names.index(stim_channel)
+    events_on = find_events(
+        raw, stim_channel=raw.ch_names[idx], output='onset',
+        verbose=False)[:, 0]
+    events_off = find_events(
+        raw, stim_channel=raw.ch_names[idx], output='offset',
+        verbose=False)[:, 0]
+    bad = False
+    if len(events_on) == 0 or len(events_off) == 0:
+        bad = True
+    else:
+        if events_on[-1] > events_off[-1]:
+            events_on = events_on[:-1]
+        if events_on.size != events_off.size or not \
+                (events_on < events_off).all():
+            bad = True
+    if bad:
+        raise RuntimeError(
+            f'Could not find appropriate cHPI intervals from {stim_channel}')
+    # use the midpoint for times
+    times = (events_on + events_off) / (2 * raw.info['sfreq'])
+    del events_on, events_off
+    # XXX remove first two rows. It is unknown currently if there is a way to
+    # determine from the con file the number of initial pulses that
+    # indicate the start of reading. The number is shown by opening the con
+    # file in MEG160, but I couldn't find the value in the .con file, so it
+    # may just always be 2...
+    times = times[2:]
+    n_coils = 5  # KIT always has 5 (hard-coded in reader)
+    header = raw._raw_extras[0]['dirs'][KIT.DIR_INDEX_CHPI_DATA]
+    dtype = np.dtype([('good', '<u4'), ('data', '<f8', (4,))])
+    assert dtype.itemsize == header['size'], (dtype.itemsize, header['size'])
+    all_data = list()
+    for fname in raw._filenames:
+        with open(fname, 'r') as fid:
+            fid.seek(header['offset'])
+            all_data.append(np.fromfile(
+                fid, dtype, count=header['count']).reshape(-1, n_coils))
+    data = np.concatenate(all_data)
+    extra = ''
+    if len(times) < len(data):
+        extra = f', truncating to {len(times)} based on events'
+    logger.info(f'Found {len(data)} cHPI measurement{_pl(len(data))}{extra}')
+    data = data[:len(times)]
+    # good is not currently used, but keep this in case we want it later
+    # good = data['good'] == 1
+    data = data['data']
+    rrs, gofs = data[:, :, :3], data[:, :, 3]
+    rrs = apply_trans(als_ras_trans, rrs)
+    moments = np.zeros(rrs.shape)  # not encoded, set all zero
+    return dict(rrs=rrs, gofs=gofs, times=times, moments=moments)
+
+
 # ############################################################################
 # Estimate positions from data
+
 @verbose
-def _get_hpi_info(info, allow_empty=False, verbose=None):
-    """Get HPI information from raw."""
+def get_chpi_info(info, on_missing='raise', verbose=None):
+    """Retrieve cHPI information from the data.
+
+    Parameters
+    ----------
+    %(info_not_none)s
+    %(chpi_on_missing)s
+    %(verbose)s
+
+    Returns
+    -------
+    hpi_freqs : array, shape (n_coils,)
+        The frequency used for each individual cHPI coil.
+    hpi_pick : int | None
+        The index of the ``STIM`` channel containing information about when
+        which cHPI coils were switched on.
+    hpi_on : array, shape (n_coils,)
+        The values coding for the "on" state of each individual cHPI coil.
+
+    Notes
+    -----
+    .. versionadded:: 0.24
+    """
+    _validate_type(item=info, item_name='info', types=Info)
+    _check_option(parameter='on_missing', value=on_missing,
+                  allowed_values=['ignore', 'raise', 'warn'])
+
     if len(info['hpi_meas']) == 0 or \
             ('coil_freq' not in info['hpi_meas'][0]['hpi_coils'][0]):
-        if allow_empty:
-            return np.empty(0), None, np.empty(0)
-        raise RuntimeError('Appropriate cHPI information not found in'
-                           'info["hpi_meas"] and info["hpi_subsystem"], '
-                           'cannot process cHPI')
+        _on_missing(on_missing,
+                    msg='No appropriate cHPI information found in '
+                        'info["hpi_meas"] and info["hpi_subsystem"]')
+        return np.empty(0), None, np.empty(0)
+
     hpi_coils = sorted(info['hpi_meas'][-1]['hpi_coils'],
                        key=lambda x: x['number'])  # ascending (info) order
 
@@ -410,7 +518,7 @@ def _fit_chpi_quat(coil_dev_rrs, coil_head_rrs):
     return quat, gof
 
 
-def _fit_coil_order_dev_head_trans(dev_pnts, head_pnts):
+def _fit_coil_order_dev_head_trans(dev_pnts, head_pnts, bias=True):
     """Compute Device to Head transform allowing for permutiatons of points."""
     id_quat = np.zeros(6)
     best_order = None
@@ -420,11 +528,14 @@ def _fit_coil_order_dev_head_trans(dev_pnts, head_pnts):
         head_pnts_tmp = head_pnts[np.array(this_order)]
         this_quat, g = _fit_chpi_quat(dev_pnts, head_pnts_tmp)
         assert np.linalg.det(quat_to_rot(this_quat[:3])) > 0.9999
-        # For symmetrical arrangements, flips can produce roughly
-        # equivalent g values. To avoid this, heavily penalize
-        # large rotations.
-        rotation = _angle_between_quats(this_quat[:3], np.zeros(3))
-        check_g = g * max(1. - rotation / np.pi, 0) ** 0.25
+        if bias:
+            # For symmetrical arrangements, flips can produce roughly
+            # equivalent g values. To avoid this, heavily penalize
+            # large rotations.
+            rotation = _angle_between_quats(this_quat[:3], np.zeros(3))
+            check_g = g * max(1. - rotation / np.pi, 0) ** 0.25
+        else:
+            check_g = g
         if check_g > best_g:
             out_g = g
             best_g = check_g
@@ -441,21 +552,17 @@ def _setup_hpi_amplitude_fitting(info, t_window, remove_aliased=False,
                                  ext_order=1, allow_empty=False, verbose=None):
     """Generate HPI structure for HPI localization."""
     # grab basic info.
-    hpi_freqs, hpi_pick, hpi_ons = _get_hpi_info(info, allow_empty=allow_empty)
+    on_missing = 'raise' if not allow_empty else 'ignore'
+    hpi_freqs, hpi_pick, hpi_ons = get_chpi_info(info, on_missing=on_missing)
+
     _validate_type(t_window, (str, 'numeric'), 't_window')
-    if isinstance(t_window, str):
-        if t_window != 'auto':
-            raise ValueError('t_window must be "auto" if a string, got %r'
-                             % (t_window,))
-        if len(hpi_freqs):
-            t_window = max(5. / min(hpi_freqs), 1. / np.diff(hpi_freqs).min())
-        else:
-            t_window = 0.2
-    t_window = float(t_window)
-    if t_window <= 0:
-        raise ValueError('t_window (%s) must be > 0' % (t_window,))
-    logger.info('Using time window: %0.1f ms' % (1000 * t_window,))
-    model_n_window = int(round(float(t_window) * info['sfreq']))
+    if info['line_freq'] is not None:
+        line_freqs = np.arange(info['line_freq'], info['sfreq'] / 3.,
+                               info['line_freq'])
+    else:
+        line_freqs = np.zeros([0])
+    logger.info('Line interference frequencies: %s Hz'
+                % ' '.join(['%d' % lf for lf in line_freqs]))
     # worry about resampled/filtered data.
     # What to do e.g. if Raw has been resampled and some of our
     # HPI freqs would now be aliased
@@ -469,37 +576,48 @@ def _setup_hpi_amplitude_fitting(info, t_window, remove_aliased=False,
         raise RuntimeError('Found HPI frequencies %s above the lowpass '
                            '(or Nyquist) frequency %0.1f'
                            % (hpi_freqs[~keepers].tolist(), highest))
-    if info['line_freq'] is not None:
-        line_freqs = np.arange(info['line_freq'], info['sfreq'] / 3.,
-                               info['line_freq'])
-    else:
-        line_freqs = np.zeros([0])
-    logger.info('Line interference frequencies: %s Hz'
-                % ' '.join(['%d' % l for l in line_freqs]))
-
-    # build model to extract sinusoidal amplitudes.
-    slope = np.linspace(-0.5, 0.5, model_n_window)[:, np.newaxis]
-    rps = np.arange(model_n_window)[:, np.newaxis].astype(float)
-    rps *= 2 * np.pi / info['sfreq']  # radians/sec
-    f_t = hpi_freqs[np.newaxis, :] * rps
-    l_t = line_freqs[np.newaxis, :] * rps
-    model = [np.sin(f_t), np.cos(f_t)]  # hpi freqs
-    model += [np.sin(l_t), np.cos(l_t)]  # line freqs
-    model += [slope, np.ones(slope.shape)]
-    model = np.concatenate(model, axis=1)
-    inv_model = linalg.pinv(model)
+    # calculate optimal window length.
+    if isinstance(t_window, str):
+        _check_option('t_window', t_window, ('auto',), extra='if a string')
+        if len(hpi_freqs):
+            all_freqs = np.concatenate((hpi_freqs, line_freqs))
+            delta_freqs = np.diff(np.unique(all_freqs))
+            t_window = max(5. / all_freqs.min(), 1. / delta_freqs.min())
+        else:
+            t_window = 0.2
+    t_window = float(t_window)
+    if t_window <= 0:
+        raise ValueError('t_window (%s) must be > 0' % (t_window,))
+    logger.info('Using time window: %0.1f ms' % (1000 * t_window,))
+    window_nsamp = np.rint(t_window * info['sfreq']).astype(int)
+    model = _setup_hpi_glm(hpi_freqs, line_freqs, info['sfreq'], window_nsamp)
+    inv_model = np.linalg.pinv(model)
     inv_model_reord = _reorder_inv_model(inv_model, len(hpi_freqs))
     proj, proj_op, meg_picks = _setup_ext_proj(info, ext_order)
-
+    # include mag and grad picks separately, for SNR computations
+    mag_picks = _picks_to_idx(info, 'mag', allow_empty=True)
+    grad_picks = _picks_to_idx(info, 'grad', allow_empty=True)
     # Set up magnetic dipole fits
-    hpi = dict(meg_picks=meg_picks, hpi_pick=hpi_pick,
-               model=model, inv_model=inv_model, t_window=t_window,
-               inv_model_reord=inv_model_reord,
-               on=hpi_ons, n_window=model_n_window, proj=proj, proj_op=proj_op,
-               freqs=hpi_freqs, line_freqs=line_freqs)
+    hpi = dict(
+        meg_picks=meg_picks, mag_picks=mag_picks, grad_picks=grad_picks,
+        hpi_pick=hpi_pick, model=model, inv_model=inv_model, t_window=t_window,
+        inv_model_reord=inv_model_reord, on=hpi_ons, n_window=window_nsamp,
+        proj=proj, proj_op=proj_op, freqs=hpi_freqs, line_freqs=line_freqs)
     return hpi
 
 
+def _setup_hpi_glm(hpi_freqs, line_freqs, sfreq, window_nsamp):
+    """Initialize a general linear model for HPI amplitude estimation."""
+    slope = np.linspace(-0.5, 0.5, window_nsamp)[:, np.newaxis]
+    radians_per_sec = 2 * np.pi * np.arange(window_nsamp, dtype=float) / sfreq
+    f_t = hpi_freqs[np.newaxis, :] * radians_per_sec[:, np.newaxis]
+    l_t = line_freqs[np.newaxis, :] * radians_per_sec[:, np.newaxis]
+    model = [np.sin(f_t), np.cos(f_t),    # hpi freqs
+             np.sin(l_t), np.cos(l_t),    # line freqs
+             slope, np.ones_like(slope)]  # drift, DC
+    return np.hstack(model)
+
+
 @jit()
 def _reorder_inv_model(inv_model, n_freqs):
     # Reorder for faster computation
@@ -508,16 +626,17 @@ def _reorder_inv_model(inv_model, n_freqs):
 
 
 def _setup_ext_proj(info, ext_order):
+    from scipy import linalg
     meg_picks = pick_types(info, meg=True, eeg=False, exclude='bads')
     info = pick_info(_simplify_info(info), meg_picks)  # makes a copy
-    _, _, _, _, mag_or_fine = _get_mf_picks(
+    _, _, _, _, mag_or_fine = _get_mf_picks_fix_mags(
         info, int_order=0, ext_order=ext_order, ignore_ref=True,
         verbose='error')
     mf_coils = _prep_mf_coils(info, verbose='error')
     ext = _sss_basis(
         dict(origin=(0., 0., 0.), int_order=0, ext_order=ext_order),
         mf_coils).T
-    out_removes = _regularize_out(0, 1, mag_or_fine)
+    out_removes = _regularize_out(0, 1, mag_or_fine, [])
     ext = ext[~np.in1d(np.arange(len(ext)), out_removes)]
     ext = linalg.orth(ext.T).T
     assert ext.shape[1] == len(meg_picks)
@@ -525,7 +644,8 @@ def _setup_ext_proj(info, ext_order):
         kind=FIFF.FIFFV_PROJ_ITEM_HOMOG_FIELD, desc='SSS', active=False,
         data=dict(data=ext, ncol=info['nchan'], col_names=info['ch_names'],
                   nrow=len(ext)))
-    info['projs'] = [proj]
+    with info._unlock():
+        info['projs'] = [proj]
     proj_op, _ = setup_proj(
         info, add_eeg_ref=False, activate=False, verbose=False)
     assert proj_op.shape == (len(meg_picks),) * 2
@@ -537,14 +657,16 @@ def _time_prefix(fit_time):
     return ('    t=%0.3f:' % fit_time).ljust(17)
 
 
-def _fit_chpi_amplitudes(raw, time_sl, hpi):
+def _fit_chpi_amplitudes(raw, time_sl, hpi, snr=False):
     """Fit amplitudes for each channel from each of the N cHPI sinusoids.
 
     Returns
     -------
-    sin_fit : ndarray, shape (n_freqs, n_channels))
+    sin_fit : ndarray, shape (n_freqs, n_channels)
         The sin amplitudes matching each cHPI frequency.
         Will be all nan if this time window should be skipped.
+    snr : ndarray, shape (n_freqs, 2)
+        Estimated SNR for this window, separately for mag and grad channels.
     """
     # No need to detrend the data because our model has a DC term
     with use_log_level(False):
@@ -557,11 +679,15 @@ def _fit_chpi_amplitudes(raw, time_sl, hpi):
             # loads hpi_stim channel
             chpi_data = raw[hpi['hpi_pick'], time_sl][0]
 
-        ons = (np.round(chpi_data).astype(np.int) &
+        ons = (np.round(chpi_data).astype(np.int64) &
                hpi['on'][:, np.newaxis]).astype(bool)
         n_on = ons.all(axis=-1).sum(axis=0)
         if not (n_on >= 3).all():
             return None
+    if snr:
+        return _fast_fit_snr(
+            this_data, len(hpi['freqs']), hpi['model'], hpi['inv_model'],
+            hpi['mag_picks'], hpi['grad_picks'])
     return _fast_fit(this_data, hpi['proj_op'], len(hpi['freqs']),
                      hpi['model'], hpi['inv_model_reord'])
 
@@ -572,8 +698,8 @@ def _fast_fit(this_data, proj, n_freqs, model, inv_model_reord):
     if this_data.shape[1] != model.shape[0]:
         model = model[:this_data.shape[1]]
         inv_model_reord = _reorder_inv_model(np.linalg.pinv(model), n_freqs)
-    proj_data = np.dot(proj, this_data)
-    X = np.dot(inv_model_reord, proj_data.T)
+    proj_data = proj @ this_data
+    X = inv_model_reord @ proj_data.T
 
     sin_fit = np.zeros((n_freqs, X.shape[1]))
     for fi in range(n_freqs):
@@ -585,6 +711,34 @@ def _fast_fit(this_data, proj, n_freqs, model, inv_model_reord):
     return sin_fit
 
 
+@jit()
+def _fast_fit_snr(this_data, n_freqs, model, inv_model, mag_picks, grad_picks):
+    # first or last window
+    if this_data.shape[1] != model.shape[0]:
+        model = model[:this_data.shape[1]]
+        inv_model = np.linalg.pinv(model)
+    coefs = np.ascontiguousarray(inv_model) @ np.ascontiguousarray(this_data.T)
+    # average sin & cos terms (special property of sinusoids: power=A²/2)
+    hpi_power = (coefs[:n_freqs] ** 2 + coefs[n_freqs:(2 * n_freqs)] ** 2) / 2
+    resid = this_data - np.ascontiguousarray((model @ coefs).T)
+    # can't use np.var(..., axis=1) with Numba, so do it manually:
+    resid_mean = np.atleast_2d(resid.sum(axis=1) / resid.shape[1]).T
+    squared_devs = np.abs(resid - resid_mean) ** 2
+    resid_var = squared_devs.sum(axis=1) / squared_devs.shape[1]
+    # output array will be (n_freqs, 3 * n_ch_types). The 3 columns for each
+    # channel type are the SNR, the mean cHPI power and the residual variance
+    # (which gets tiled to shape (n_freqs,) because it's a scalar).
+    snrs = np.empty((n_freqs, 0))
+    # average power & compute residual variance separately for each ch type
+    for _picks in (mag_picks, grad_picks):
+        if len(_picks):
+            avg_power = hpi_power[:, _picks].sum(axis=1) / len(_picks)
+            avg_resid = resid_var[_picks].mean() * np.ones(n_freqs)
+            snr = 10 * np.log10(avg_power / avg_resid)
+            snrs = np.hstack((snrs, np.stack((snr, avg_power, avg_resid), 1)))
+    return snrs
+
+
 def _check_chpi_param(chpi_, name):
     if name == 'chpi_locs':
         want_ndims = dict(times=1, rrs=3, moments=3, gofs=2)
@@ -643,8 +797,7 @@ def compute_head_pos(info, chpi_locs, dist_limit=0.005, gof_limit=0.98,
 
     Parameters
     ----------
-    info : instance of Info
-        Measurement information.
+    %(info_not_none)s
     %(chpi_locs)s
         Typically obtained by :func:`~mne.chpi.compute_chpi_locs` or
         :func:`~mne.chpi.extract_chpi_locs_ctf`.
@@ -672,6 +825,7 @@ def compute_head_pos(info, chpi_locs, dist_limit=0.005, gof_limit=0.98,
     .. versionadded:: 0.20
     """
     _check_chpi_param(chpi_locs, 'chpi_locs')
+    _validate_type(info, Info, 'info')
     hpi_dig_head_rrs = _get_hpi_initial_fit(info, adjust=adjust_dig,
                                             verbose='error')
     n_coils = len(hpi_dig_head_rrs)
@@ -693,7 +847,7 @@ def compute_head_pos(info, chpi_locs, dist_limit=0.005, gof_limit=0.98,
         #
         if len(use_idx) < 3:
             msg = (_time_prefix(fit_time) + '%s/%s good HPI fits, cannot '
-                   'determine the transformation (%s)!'
+                   'determine the transformation (%s GOF)!'
                    % (len(use_idx), n_coils,
                       ', '.join('%0.2f' % g for g in g_coils)))
             warn(msg)
@@ -718,8 +872,10 @@ def compute_head_pos(info, chpi_locs, dist_limit=0.005, gof_limit=0.98,
         n_good = ((g_coils >= gof_limit) & (errs < dist_limit)).sum()
         if n_good < 3:
             warn(_time_prefix(fit_time) + '%s/%s good HPI fits, cannot '
-                 'determine the transformation (%s)!'
-                 % (n_good, n_coils, ', '.join('%0.2f' % g for g in g_coils)))
+                 'determine the transformation (%s mm/GOF)!'
+                 % (n_good, n_coils,
+                    ', '.join(f'{1000 * e:0.1f}::{g:0.2f}'
+                              for e, g in zip(errs, g_coils))))
             continue
 
         # velocities, in device coords, of HPI coils
@@ -797,6 +953,43 @@ def _unit_quat_constraint(x):
     return 1 - (x * x).sum()
 
 
+@verbose
+def compute_chpi_snr(raw, t_step_min=0.01, t_window='auto', ext_order=1,
+                     tmin=0, tmax=None, verbose=None):
+    """Compute time-varying estimates of cHPI SNR.
+
+    Parameters
+    ----------
+    raw : instance of Raw
+        Raw data with cHPI information.
+    t_step_min : float
+        Minimum time step to use.
+    %(chpi_t_window)s
+    %(chpi_ext_order)s
+    %(raw_tmin)s
+    %(raw_tmax)s
+    %(verbose)s
+
+    Returns
+    -------
+    chpi_snrs : dict
+        The time-varying cHPI SNR estimates, with entries "times", "freqs",
+        "snr_mag", "power_mag", and "resid_mag" (and/or "snr_grad",
+        "power_grad", and "resid_grad", depending on which channel types are
+        present in ``raw``).
+
+    See Also
+    --------
+    mne.chpi.compute_chpi_locs, mne.chpi.compute_chpi_amplitudes
+
+    Notes
+    -----
+    .. versionadded:: 0.24
+    """
+    return _compute_chpi_amp_or_snr(raw, t_step_min, t_window, ext_order,
+                                    tmin, tmax, verbose, snr=True)
+
+
 @verbose
 def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
                             ext_order=1, tmin=0, tmax=None, verbose=None):
@@ -807,8 +1000,7 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
     raw : instance of Raw
         Raw data with cHPI information.
     t_step_min : float
-        Minimum time step to use. If correlations are sufficiently high,
-        t_step_max will be used.
+        Minimum time step to use.
     %(chpi_t_window)s
     %(chpi_ext_order)s
     %(raw_tmin)s
@@ -821,7 +1013,7 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
 
     See Also
     --------
-    mne.chpi.compute_chpi_locs
+    mne.chpi.compute_chpi_locs, mne.chpi.compute_chpi_snr
 
     Notes
     -----
@@ -835,10 +1027,30 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
        sinusoidal amplitudes to MEG channels.
        It uses SVD to determine the phase/amplitude of the sinusoids.
 
+    In "auto" mode, ``t_window`` will be set to the longer of:
+
+    1. Five cycles of the lowest HPI or line frequency.
+          Ensures that the frequency estimate is stable.
+    2. The reciprocal of the smallest difference between HPI and line freqs.
+          Ensures that neighboring frequencies can be disambiguated.
+
     The output is meant to be used with :func:`~mne.chpi.compute_chpi_locs`.
 
     .. versionadded:: 0.20
     """
+    return _compute_chpi_amp_or_snr(raw, t_step_min, t_window, ext_order,
+                                    tmin, tmax, verbose)
+
+
+def _compute_chpi_amp_or_snr(raw, t_step_min=0.01, t_window='auto',
+                             ext_order=1, tmin=0, tmax=None, verbose=None,
+                             snr=False):
+    """Compute cHPI amplitude or SNR.
+
+    See compute_chpi_amplitudes for parameter descriptions. One additional
+    boolean parameter ``snr`` signals whether to return SNR instead of
+    amplitude.
+    """
     hpi = _setup_hpi_amplitude_fitting(raw.info, t_window, ext_order=ext_order)
     tmin, tmax = raw._tmin_tmax_to_start_stop(tmin, tmax)
     tmin = tmin / raw.info['sfreq']
@@ -851,14 +1063,26 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
                 % (len(hpi['freqs']), len(fit_idxs), tmax - tmin))
     del tmin, tmax
     sin_fits = dict()
-    sin_fits['times'] = (fit_idxs + raw.first_samp -
-                         hpi['n_window'] / 2.) / raw.info['sfreq']
     sin_fits['proj'] = hpi['proj']
-    sin_fits['slopes'] = np.empty(
-        (len(sin_fits['times']),
-         len(hpi['freqs']),
-         len(sin_fits['proj']['data']['col_names'])))
-    for mi, midpt in enumerate(ProgressBar(fit_idxs, mesg='cHPI amplitudes')):
+    sin_fits['times'] = np.round(fit_idxs + raw.first_samp -
+                                 hpi['n_window'] / 2.) / raw.info['sfreq']
+    n_times = len(sin_fits['times'])
+    n_freqs = len(hpi['freqs'])
+    n_chans = len(sin_fits['proj']['data']['col_names'])
+    if snr:
+        del sin_fits['proj']
+        sin_fits['freqs'] = hpi['freqs']
+        ch_types = raw.get_channel_types()
+        grad_offset = 3 if 'mag' in ch_types else 0
+        for ch_type in ('mag', 'grad'):
+            if ch_type in ch_types:
+                for key in ('snr', 'power', 'resid'):
+                    cols = 1 if key == 'resid' else n_freqs
+                    sin_fits[f'{ch_type}_{key}'] = np.empty((n_times, cols))
+    else:
+        sin_fits['slopes'] = np.empty((n_times, n_freqs, n_chans))
+    message = f"cHPI {'SNRs' if snr else 'amplitudes'}"
+    for mi, midpt in enumerate(ProgressBar(fit_idxs, mesg=message)):
         #
         # 0. determine samples to fit.
         #
@@ -869,7 +1093,24 @@ def compute_chpi_amplitudes(raw, t_step_min=0.01, t_window='auto',
         #
         # 1. Fit amplitudes for each channel from each of the N sinusoids
         #
-        sin_fits['slopes'][mi] = _fit_chpi_amplitudes(raw, time_sl, hpi)
+        amps_or_snrs = _fit_chpi_amplitudes(raw, time_sl, hpi, snr)
+        if snr:
+            # unpack the SNR estimates. mag & grad are returned in one array
+            # (because of Numba) so take care with which column is which.
+            # note that mean residual is a scalar (same for all HPI freqs) but
+            # is returned as a (tiled) vector (again, because Numba) so that's
+            # why below we take amps_or_snrs[0, 2] instead of [:, 2]
+            ch_types = raw.get_channel_types()
+            if 'mag' in ch_types:
+                sin_fits['mag_snr'][mi] = amps_or_snrs[:, 0]    # SNR
+                sin_fits['mag_power'][mi] = amps_or_snrs[:, 1]  # mean power
+                sin_fits['mag_resid'][mi] = amps_or_snrs[0, 2]  # mean resid
+            if 'grad' in ch_types:
+                sin_fits['grad_snr'][mi] = amps_or_snrs[:, grad_offset]
+                sin_fits['grad_power'][mi] = amps_or_snrs[:, grad_offset + 1]
+                sin_fits['grad_resid'][mi] = amps_or_snrs[0, grad_offset + 2]
+        else:
+            sin_fits['slopes'][mi] = amps_or_snrs
     return sin_fits
 
 
@@ -880,8 +1121,7 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information.
+    %(info_not_none)s
     %(chpi_amplitudes)s
         Typically obtained by :func:`mne.chpi.compute_chpi_amplitudes`.
     t_step_max : float
@@ -918,25 +1158,20 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
     movements as well as ``t_step_max`` (and ``t_step_min`` from
     :func:`mne.chpi.compute_chpi_amplitudes`).
 
-    In "auto" mode, ``t_window`` will be set to the longer of:
-
-    1. Five cycles of the lowest HPI frequency.
-          Ensures that the frequency estimate is stable.
-    2. The reciprocal of the smallest difference between HPI frequencies.
-          Ensures that neighboring frequencies can be disambiguated.
-
     .. versionadded:: 0.20
     """
     # Set up magnetic dipole fits
     _check_option('too_close', too_close, ['raise', 'warning', 'info'])
     _check_chpi_param(chpi_amplitudes, 'chpi_amplitudes')
+    _validate_type(info, Info, 'info')
     sin_fits = chpi_amplitudes  # use the old name below
     del chpi_amplitudes
     proj = sin_fits['proj']
     meg_picks = pick_channels(
         info['ch_names'], proj['data']['col_names'], ordered=True)
     info = pick_info(info, meg_picks)  # makes a copy
-    info['projs'] = [proj]
+    with info._unlock():
+        info['projs'] = [proj]
     del meg_picks, proj
     meg_coils = _concatenate_coils(_create_meg_coils(info['chs'], 'accurate'))
 
@@ -965,6 +1200,7 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
         _get_hpi_initial_fit(info, adjust=adjust_dig))
     last = dict(sin_fit=None, coil_fit_time=sin_fits['times'][0] - 1,
                 coil_dev_rrs=hpi_dig_dev_rrs)
+    n_hpi = len(hpi_dig_dev_rrs)
     del hpi_dig_dev_rrs
     for fit_time, sin_fit in ProgressBar(iter_, mesg='cHPI locations '):
         # skip this window if bad
@@ -1000,8 +1236,15 @@ def compute_chpi_locs(info, chpi_amplitudes, t_step_max=1., too_close='raise',
         chpi_locs['moments'].append(moments)
         last['coil_fit_time'] = fit_time
         last['coil_dev_rrs'] = rrs
+    n_times = len(chpi_locs['times'])
+    shapes = dict(
+        times=(n_times,),
+        rrs=(n_times, n_hpi, 3),
+        gofs=(n_times, n_hpi),
+        moments=(n_times, n_hpi, 3),
+    )
     for key, val in chpi_locs.items():
-        chpi_locs[key] = np.array(val, float)
+        chpi_locs[key] = np.array(val, float).reshape(shapes[key])
     return chpi_locs
 
 
@@ -1017,7 +1260,7 @@ def _chpi_locs_to_times_dig(chpi_locs):
 
 
 @verbose
-def filter_chpi(raw, include_line=True, t_step=0.01, t_window=None,
+def filter_chpi(raw, include_line=True, t_step=0.01, t_window='auto',
                 ext_order=1, allow_line_only=False, verbose=None):
     """Remove cHPI and line noise from data.
 
@@ -1058,11 +1301,6 @@ def filter_chpi(raw, include_line=True, t_step=0.01, t_window=None,
     _validate_type(raw, BaseRaw, 'raw')
     if not raw.preload:
         raise RuntimeError('raw data must be preloaded')
-    if t_window is None:
-        warn('The default for t_window is 0.2 in MNE 0.20 but will change '
-             'to "auto" in 0.21, set it explicitly to avoid this warning',
-             DeprecationWarning)
-        t_window = 0.2
     t_step = float(t_step)
     if t_step <= 0:
         raise ValueError('t_step (%s) must be > 0' % (t_step,))
@@ -1100,7 +1338,7 @@ def filter_chpi(raw, include_line=True, t_step=0.01, t_window=None,
             this_recon = recon
         else:  # first or last window
             model = hpi['model'][:this_len]
-            inv_model = linalg.pinv(model)
+            inv_model = np.linalg.pinv(model)
             this_recon = np.dot(model[:, :n_remove], inv_model[:n_remove]).T
         this_data = raw._data[meg_picks, time_sl]
         subt_pt = min(midpt + n_step, n_times)
