@@ -4,14 +4,14 @@
 #          Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Jean-Remi King <jeanremi.king@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import numpy as np
 import time
 import numbers
 from ..parallel import parallel_func
 from ..fixes import BaseEstimator, is_classifier, _get_check_scoring
-from ..utils import check_version, logger, warn, fill_doc
+from ..utils import logger, warn, fill_doc
 
 
 class LinearModel(BaseEstimator):
@@ -54,10 +54,7 @@ class LinearModel(BaseEstimator):
     def __init__(self, model=None):  # noqa: D102
         if model is None:
             from sklearn.linear_model import LogisticRegression
-            if check_version('sklearn', '0.20'):
-                model = LogisticRegression(solver='liblinear')
-            else:
-                model = LogisticRegression()
+            model = LogisticRegression(solver='liblinear')
 
         self.model = model
         self._estimator_type = getattr(model, "_estimator_type", None)
@@ -147,7 +144,6 @@ class LinearModel(BaseEstimator):
         -------
         y_pred : array, shape (n_samples,)
             The predicted targets.
-
         """
         return self.fit(X, y).transform(X)
 
@@ -235,12 +231,8 @@ def _set_cv(cv, estimator=None, X=None, y=None):
     cv = check_cv(cv=cv, y=y, classifier=est_is_classifier)
 
     # Extract train and test set to retrieve them at predict time
-    if hasattr(cv, 'split'):
-        cv_splits = [(train, test) for train, test in
-                     cv.split(X=np.zeros_like(y), y=y)]
-    else:
-        # XXX support sklearn.cross_validation cv
-        cv_splits = [(train, test) for train, test in cv]
+    cv_splits = [(train, test) for train, test in
+                 cv.split(X=np.zeros_like(y), y=y)]
 
     if not np.all([len(train) for train, _ in cv_splits]):
         raise ValueError('Some folds do not have any train epochs.')
