@@ -7,13 +7,13 @@ from functools import partial
 import numpy as np
 
 from .annotations import _annotations_starts_stops
+from .fixes import _import_fft
 from .io.pick import _picks_to_idx
 from .cuda import (_setup_cuda_fft_multiply_repeated, _fft_multiply_repeated,
                    _setup_cuda_fft_resample, _fft_resample, _smart_pad)
-from .fixes import irfft, ifftshift, fftfreq
 from .parallel import parallel_func, check_n_jobs
 from .time_frequency.multitaper import _mt_spectra, _compute_mt_params
-from .utils import (logger, verbose, sum_squared, check_version, warn, _pl,
+from .utils import (logger, verbose, sum_squared, warn, _pl,
                     _check_preload, _validate_type, _check_option, _ensure_int)
 from ._ola import _COLA
 
@@ -1205,7 +1205,7 @@ def _get_window_thresh(n_times, sfreq, mt_bandwidth, p_value):
     # but if we have a new enough scipy,
     # it's only ~0.175 sec for 8 tapers even with 100000 samples
     from scipy import stats
-    dpss_n_times_max = 100000 if check_version('scipy', '1.1') else 1000
+    dpss_n_times_max = 100000
 
     # figure out what tapers to use
     window_fun, _, _ = _compute_mt_params(
@@ -1224,10 +1224,7 @@ def _mt_spectrum_proc(x, sfreq, line_freqs, notch_widths, mt_bandwidth,
     n_jobs = check_n_jobs(n_jobs)
     x, orig_shape, picks = _prep_for_filtering(x, copy, picks)
     if isinstance(filter_length, str) and filter_length == 'auto':
-        filter_length = None
-        warn('The default for "filter_length" when using method="spectrum_fit"'
-             ' is None in 0.21 but will change to 10. in 0.22, set it '
-             'explicitly to avoid this warning', DeprecationWarning)
+        filter_length = '10s'
     if filter_length is None:
         filter_length = x.shape[-1]
     filter_length = min(_to_samples(filter_length, sfreq, '', ''), x.shape[-1])
@@ -1383,7 +1380,7 @@ def _check_filterable(x, kind='filtered'):
 
 def _resamp_ratio_len(up, down, n):
     ratio = float(up) / down
-    return ratio, int(round(ratio * n))
+    return ratio, max(int(round(ratio * n)), 1)
 
 
 @verbose
@@ -1431,6 +1428,7 @@ def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
     up=up/down and down=1.
     """
     from scipy.signal import get_window
+    ifftshift, fftfreq = _import_fft(('ifftshift', 'fftfreq'))
     # check explicitly for backwards compatibility
     if not isinstance(axis, int):
         err = ("The axis parameter needs to be an integer (got %s). "
@@ -1471,7 +1469,7 @@ def resample(x, up=1., down=1., npad=100, axis=-1, window='boxcar', n_jobs=1,
     # prep for resampling now
     x_flat = x.reshape((-1, x_len))
     orig_len = x_len + npads.sum()  # length after padding
-    new_len = int(round(ratio * orig_len))  # length after resampling
+    new_len = max(int(round(ratio * orig_len)), 1)  # length after resampling
     to_removes = [int(round(ratio * npads[0]))]
     to_removes.append(new_len - final_len - to_removes[0])
     to_removes = np.array(to_removes)
@@ -2021,6 +2019,9 @@ class FilterMixin(object):
                  pad='edge', verbose=None):  # lgtm
         """Resample data.
 
+        If appropriate, an anti-aliasing filter is applied before resampling.
+        See :ref:`resampling-and-decimating` for more information.
+
         .. note:: Data must be loaded.
 
         Parameters
@@ -2063,10 +2064,11 @@ class FilterMixin(object):
         o_sfreq = self.info['sfreq']
         self._data = resample(self._data, sfreq, o_sfreq, npad, window=window,
                               n_jobs=n_jobs, pad=pad)
-        self.info['sfreq'] = float(sfreq)
         lowpass = self.info.get('lowpass')
         lowpass = np.inf if lowpass is None else lowpass
-        self.info['lowpass'] = min(lowpass, sfreq / 2.)
+        with self.info._unlock():
+            self.info['lowpass'] = min(lowpass, sfreq / 2.)
+            self.info['sfreq'] = float(sfreq)
         new_times = (np.arange(self._data.shape[-1], dtype=np.float64) /
                      sfreq + self.times[0])
         # adjust indirectly affected variables
@@ -2255,6 +2257,7 @@ def design_mne_c_filter(sfreq, l_freq=None, h_freq=40.,
     4197 frequencies are directly constructed, with zeroes in the stop-band
     and ones in the passband, with squared cosine ramps in between.
     """
+    irfft = _import_fft('irfft')
     n_freqs = (4096 + 2 * 2048) // 2 + 1
     freq_resp = np.ones(n_freqs)
     l_freq = 0 if l_freq is None else float(l_freq)
@@ -2320,7 +2323,9 @@ def _filt_update_info(info, update_info, l_freq, h_freq):
     if update_info:
         if h_freq is not None and (l_freq is None or l_freq < h_freq) and \
                 (info["lowpass"] is None or h_freq < info['lowpass']):
-            info['lowpass'] = float(h_freq)
+            with info._unlock():
+                info['lowpass'] = float(h_freq)
         if l_freq is not None and (h_freq is None or l_freq < h_freq) and \
                 (info["highpass"] is None or l_freq > info['highpass']):
-            info['highpass'] = float(l_freq)
+            with info._unlock():
+                info['highpass'] = float(l_freq)
