@@ -12,7 +12,6 @@ at which the fix is no longer needed.
 #          Lars Buitinck <L.J.Buitinck@uva.nl>
 # License: BSD
 
-from distutils.version import LooseVersion
 import functools
 import inspect
 from math import log
@@ -23,13 +22,46 @@ import warnings
 import numpy as np
 
 
+###############################################################################
+# distutils
+
+# distutils has been deprecated since Python 3.10 and is scheduled for removal
+# from the standard library with the release of Python 3.12. For version
+# comparisons, we use setuptools's `parse_version` if available.
+
+def _compare_version(version_a, operator, version_b):
+    """Compare two version strings via a user-specified operator.
+
+    Parameters
+    ----------
+    version_a : str
+        First version string.
+    operator : '==' | '>' | '<' | '>=' | '<='
+        Operator to compare ``version_a`` and ``version_b`` in the form of
+        ``version_a operator version_b``.
+    version_b : str
+        Second version string.
+
+    Returns
+    -------
+    bool
+        The result of the version comparison.
+    """
+    try:
+        from pkg_resources import parse_version as parse
+    except ImportError:
+        from distutils.version import LooseVersion as parse
+
+    return eval(f'parse("{version_a}") {operator} parse("{version_b}")')
+
+
 ###############################################################################
 # Misc
 
 def _median_complex(data, axis):
     """Compute marginal median on complex data safely.
 
-    XXX: Can be removed when numpy introduces a fix.
+    Can be removed when numpy introduces a fix.
     See: https://github.com/scipy/scipy/pull/12676/.
     """
     # np.median must be passed real arrays for the desired result
@@ -147,7 +179,7 @@ def _read_geometry(filepath, read_metadata=False, read_stamp=False):
         else:
             raise ValueError("File does not appear to be a Freesurfer surface")
 
-    coords = coords.astype(np.float64)  # XXX: due to mayavi bug on mac 32bits
+    coords = coords.astype(np.float64)
 
     ret = (coords, faces)
     if read_metadata:
@@ -892,9 +924,9 @@ def svd(a, hermitian=False):
         sgn = np.sign(s)
         s = np.abs(s)
         sidx = np.argsort(s)[..., ::-1]
-        sgn = take_along_axis(sgn, sidx, axis=-1)
-        s = take_along_axis(s, sidx, axis=-1)
-        u = take_along_axis(u, sidx[..., None, :], axis=-1)
+        sgn = np.take_along_axis(sgn, sidx, axis=-1)
+        s = np.take_along_axis(s, sidx, axis=-1)
+        u = np.take_along_axis(u, sidx[..., None, :], axis=-1)
         # singular values are unsigned, move the sign into v
         vt = (u * sgn[..., np.newaxis, :]).swapaxes(-2, -1).conj()
         np.abs(s, out=s)
@@ -903,58 +935,6 @@ def svd(a, hermitian=False):
         return np.linalg.svd(a)
 
 
-###############################################################################
-# NumPy einsum backward compat (allow "optimize" arg and fix 1.14.0 bug)
-# XXX eventually we should hand-tune our `einsum` calls given our array sizes!
-
-def einsum(*args, **kwargs):
-    if 'optimize' not in kwargs:
-        kwargs['optimize'] = False
-    return np.einsum(*args, **kwargs)
-
-
-try:
-    from numpy import take_along_axis
-except ImportError:  # NumPy < 1.15
-    def take_along_axis(arr, indices, axis):
-        # normalize inputs
-        if axis is None:
-            arr = arr.flat
-            arr_shape = (len(arr),)  # flatiter has no .shape
-            axis = 0
-        else:
-            # there is a NumPy function for this, but rather than copy our
-            # internal uses should be correct, so just normalize quickly
-            if axis < 0:
-                axis += arr.ndim
-            assert 0 <= axis < arr.ndim
-            arr_shape = arr.shape
-
-        # use the fancy index
-        return arr[_make_along_axis_idx(arr_shape, indices, axis)]
-
-    def _make_along_axis_idx(arr_shape, indices, axis):
-        # compute dimensions to iterate over
-        if not np.issubdtype(indices.dtype, np.integer):
-            raise IndexError('`indices` must be an integer array')
-        if len(arr_shape) != indices.ndim:
-            raise ValueError(
-                "`indices` and `arr` must have the same number of dimensions")
-        shape_ones = (1,) * indices.ndim
-        dest_dims = list(range(axis)) + [None] + list(range(axis+1, indices.ndim))
-
-        # build a fancy index, consisting of orthogonal aranges, with the
-        # requested index inserted at the right location
-        fancy_index = []
-        for dim, n in zip(dest_dims, arr_shape):
-            if dim is None:
-                fancy_index.append(indices)
-            else:
-                ind_shape = shape_ones[:dim] + (-1,) + shape_ones[dim+1:]
-                fancy_index.append(np.arange(n).reshape(ind_shape))
-
-        return tuple(fancy_index)
-
 ###############################################################################
 # From nilearn
 
@@ -980,9 +960,9 @@ def _crop_colorbar(cbar, cbar_vmin, cbar_vmax):
     # _outline was removed in
     # https://github.com/matplotlib/matplotlib/commit/03a542e875eba091a027046d5ec652daa8be6863
     # so we use the code from there
-    if LooseVersion(matplotlib.__version__) >= LooseVersion("3.2.0"):
+    if _compare_version(matplotlib.__version__, '>=', '3.2.0'):
         cbar.ax.set_ylim(cbar_vmin, cbar_vmax)
-        X, _ = cbar._mesh()
+        X = cbar._mesh()[0]
         X = np.array([X[0], X[-1]])
         Y = np.array([[cbar_vmin, cbar_vmin], [cbar_vmax, cbar_vmax]])
         N = X.shape[0]
@@ -1001,18 +981,8 @@ def _crop_colorbar(cbar, cbar_vmin, cbar_vmax):
         outline[6:, 1] += cbar.norm(cbar_vmin)
         cbar.outline.set_xy(outline)
 
-    cbar.set_ticks(new_tick_locs, update_ticks=True)
-
-
-###############################################################################
-# Matplotlib
-
-def _get_status(checks):
-    """Deal with old MPL to get check box statuses."""
-    try:
-        return list(checks.get_status())
-    except AttributeError:
-        return [x[0].get_visible() for x in checks.lines]
+    cbar.set_ticks(new_tick_locs)
+    cbar.update_ticks()
 
 
 ###############################################################################
@@ -1021,7 +991,7 @@ def _get_status(checks):
 # Here we choose different defaults to speed things up by default
 try:
     import numba
-    if LooseVersion(numba.__version__) < LooseVersion('0.40'):
+    if _compare_version(numba.__version__, '<', '0.40'):
         raise ImportError
     prange = numba.prange
     def jit(nopython=True, nogil=True, fastmath=True, cache=True,
@@ -1072,13 +1042,69 @@ else:
 
 
 ###############################################################################
-# Added in Python 3.7 (remove when we drop support for 3.6)
+# workaround: plt.close() doesn't spawn close_event on Agg backend
+# (check MPL github issue #18609; scheduled to be fixed by MPL 3.4)
 
-try:
-    from contextlib import nullcontext
-except ImportError:
-    from contextlib import contextmanager
+def _close_event(fig):
+    """Force calling of the MPL figure close event."""
+    try:
+        fig.canvas.close_event()
+    except ValueError:  # old mpl with Qt
+        pass  # pragma: no cover
 
-    @contextmanager
-    def nullcontext(enter_result=None):
-        yield enter_result
+
+def _is_last_row(ax):
+    try:
+        return ax.get_subplotspec().is_last_row()  # 3.4+
+    except AttributeError:
+        return ax.is_last_row()
+    return ax.get_subplotspec().is_last_row()
+
+
+###############################################################################
+# SciPy deprecation of pinv + pinvh rcond (never worked properly anyway) in 1.7
+
+def pinvh(a, rtol=None):
+    """Compute a pseudo-inverse of a Hermitian matrix."""
+    s, u = np.linalg.eigh(a)
+    del a
+    if rtol is None:
+        rtol = s.size * np.finfo(s.dtype).eps
+    maxS = np.max(np.abs(s))
+    above_cutoff = (abs(s) > maxS * rtol)
+    psigma_diag = 1.0 / s[above_cutoff]
+    u = u[:, above_cutoff]
+    return (u * psigma_diag) @ u.conj().T
+
+
+def pinv(a, rtol=None):
+    """Compute a pseudo-inverse of a matrix."""
+    u, s, vh = np.linalg.svd(a, full_matrices=False)
+    del a
+    maxS = np.max(s)
+    if rtol is None:
+        rtol = max(vh.shape + u.shape) * np.finfo(u.dtype).eps
+    rank = np.sum(s > maxS * rtol)
+    u = u[:, :rank]
+    u /= s[:rank]
+    return (u @ vh[:rank]).conj().T
+
+
+###############################################################################
+# PyVista
+
+# Deal with pyvista deprecation of point_data and cell_data
+# (can be removed once we require 0.31+)
+
+def _point_data(obj):
+    try:
+        return obj.point_data
+    except AttributeError:
+        return obj.point_arrays
+
+
+def _cell_data(obj):
+    try:
+        return obj.cell_data
+    except AttributeError:
+        return obj.cell_arrays
