@@ -3,7 +3,7 @@
 #          Martin Luessi <mluessi@nmr.mgh.harvard.edu>
 #          Eric Larson <larsoner@uw.edu>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 # The computations in this code were primarily derived from Matti Hämäläinen's
 # C code.
@@ -21,13 +21,13 @@ from ..io.pick import _has_kit_refs, pick_types, pick_info
 from ..io.constants import FIFF, FWD
 from ..transforms import (_ensure_trans, transform_surface_to, apply_trans,
                           _get_trans, _print_coord_trans, _coord_frame_name,
-                          Transform)
+                          Transform, invert_transform)
 from ..utils import logger, verbose, warn, _pl
 from ..parallel import check_n_jobs
 from ..source_space import (_ensure_src, _filter_source_spaces,
                             _make_discrete_source_space, _complete_vol_src)
 from ..source_estimate import VolSourceEstimate
-from ..surface import _normalize_vectors
+from ..surface import _normalize_vectors, _CheckInside
 from ..bem import read_bem_solution, _bem_find_surface, ConductorModel
 
 from .forward import Forward, _merge_meg_eeg_fwds, convert_forward_solution
@@ -105,7 +105,10 @@ def _read_coil_def_file(fname, use_registry=True):
                 while(line[0] == '#'):
                     line = lines.pop()
                 vals = np.fromstring(line, sep=' ')
-                assert len(vals) == 7
+                if len(vals) != 7:
+                    raise RuntimeError(
+                        f'Could not interpret line {p + 1} as 7 points:\n'
+                        f'{line}')
                 # Read and verify data for each integration point
                 w.append(vals[0])
                 rmag.append(vals[[1, 2, 3]])
@@ -263,18 +266,16 @@ def _setup_bem(bem, bem_extra, neeg, mri_head_t, allow_none=False,
 
 
 @verbose
-def _prep_meg_channels(info, accurate=True, exclude=(), ignore_ref=False,
+def _prep_meg_channels(info, accuracy='accurate', exclude=(), ignore_ref=False,
                        head_frame=True, do_es=False, do_picking=True,
                        verbose=None):
     """Prepare MEG coil definitions for forward calculation.
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information dictionary
-    accurate : bool
-        If true (default) then use `accurate` coil definitions (more
-        integration points)
+    %(info_not_none)s
+    accuracy : str
+        Can be "normal" or "accurate" (default).
     exclude : list of str | str
         List of channels to exclude. If 'bads', exclude channels in
         info['bads']
@@ -299,7 +300,6 @@ def _prep_meg_channels(info, accurate=True, exclude=(), ignore_ref=False,
     meginfo : instance of Info
         Information subselected for just the set of MEG coils
     """
-    accuracy = 'accurate' if accurate else 'normal'
     info_extra = 'info'
     megnames, megcoils, compcoils = [], [], []
 
@@ -378,8 +378,7 @@ def _prep_eeg_channels(info, exclude=(), verbose=None):
 
     Parameters
     ----------
-    info : instance of Info
-        The measurement information dictionary
+    %(info_not_none)s
     exclude : list of str | str
         List of channels to exclude. If 'bads', exclude channels in
         info['bads']
@@ -457,7 +456,7 @@ def _prepare_for_forward(src, mri_head_t, info, bem, mindist, n_jobs,
     megcoils, compcoils, megnames, meg_info = [], [], [], []
     eegels, eegnames = [], []
 
-    if meg and len(pick_types(info, ref_meg=False, exclude=[])) > 0:
+    if meg and len(pick_types(info, meg=True, ref_meg=False, exclude=[])) > 0:
         megcoils, compcoils, megnames, meg_info = \
             _prep_meg_channels(info, ignore_ref=ignore_ref)
     if eeg and len(pick_types(info, meg=False, eeg=True, ref_meg=False,
@@ -483,11 +482,38 @@ def _prepare_for_forward(src, mri_head_t, info, bem, mindist, n_jobs,
     bem = _setup_bem(bem, bem_extra, len(eegnames), mri_head_t,
                      allow_none=allow_bem_none)
 
-    # Circumvent numerical problems by excluding points too close to the skull
-    if bem is not None and not bem['is_sphere']:
-        inner_skull = _bem_find_surface(bem, 'inner_skull')
-        _filter_source_spaces(inner_skull, mindist, mri_head_t, src, n_jobs)
-        logger.info('')
+    # Circumvent numerical problems by excluding points too close to the skull,
+    # and check that sensors are not inside any BEM surface
+    if bem is not None:
+        if not bem['is_sphere']:
+            check_surface = 'inner skull surface'
+            inner_skull = _bem_find_surface(bem, 'inner_skull')
+            check_inside = _filter_source_spaces(
+                inner_skull, mindist, mri_head_t, src, n_jobs)
+            logger.info('')
+            if len(bem['surfs']) == 3:
+                check_surface = 'scalp surface'
+                check_inside = _CheckInside(
+                    _bem_find_surface(bem, 'head'))
+        else:
+            check_surface = 'outermost sphere shell'
+            if len(bem['layers']) == 0:
+                def check_inside(x):
+                    return np.zeros(len(x), bool)
+            else:
+                def check_inside(x):
+                    return (np.linalg.norm(x, axis=1) <
+                            bem['layers'][-1]['rad'])
+        if len(megcoils):
+            meg_loc = apply_trans(
+                invert_transform(mri_head_t),
+                np.array([coil['r0'] for coil in megcoils]))
+            n_inside = check_inside(meg_loc).sum()
+            if n_inside:
+                raise RuntimeError(
+                    f'Found {n_inside} MEG sensor{_pl(n_inside)} inside the '
+                    f'{check_surface}, perhaps coordinate frames and/or '
+                    'coregistration must be incorrect')
 
     rr = np.concatenate([s['rr'][s['vertno']] for s in src])
     if len(rr) < 1:
@@ -511,10 +537,7 @@ def make_forward_solution(info, trans, src, bem, meg=True, eeg=True,
 
     Parameters
     ----------
-    info : instance of mne.Info | str
-        If str, then it should be a filename to a Raw, Epochs, or Evoked
-        file with measurement information. If dict, should be an info
-        dict (such as one from Raw, Epochs, or Evoked).
+    %(info_str)s
     %(trans)s
     src : str | instance of SourceSpaces
         If string, should be a source space filename. Can also be an
@@ -690,7 +713,7 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
     # Check for omissions due to proximity to inner skull in
     # make_forward_solution, which will result in an exception
     if fwd['src'][0]['nuse'] != len(pos):
-        inuse = fwd['src'][0]['inuse'].astype(np.bool)
+        inuse = fwd['src'][0]['inuse'].astype(bool)
         head = ('The following dipoles are outside the inner skull boundary')
         msg = len(head) * '#' + '\n' + head + '\n'
         for (t, pos) in zip(times[np.logical_not(inuse)],
@@ -728,14 +751,14 @@ def make_forward_dipole(dipole, bem, info, trans=None, n_jobs=1, verbose=None):
         row += len(amp)
 
     if tstep > 0:
-        stc = VolSourceEstimate(data, vertices=fwd['src'][0]['vertno'],
+        stc = VolSourceEstimate(data, vertices=[fwd['src'][0]['vertno']],
                                 tmin=timepoints[0],
                                 tstep=tstep, subject=None)
     else:  # Must return a list of stc, one for each time point
         stc = []
         for col, tp in enumerate(timepoints):
             stc += [VolSourceEstimate(data[:, col][:, np.newaxis],
-                                      vertices=fwd['src'][0]['vertno'],
+                                      vertices=[fwd['src'][0]['vertno']],
                                       tmin=tp, tstep=0.001, subject=None)]
     return fwd, stc
 
