@@ -2,7 +2,7 @@
 
 # Author: Eric Larson <larson.eric.d<gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from time import strptime
 from calendar import timegm
@@ -134,17 +134,12 @@ def _at_origin(x):
 
 
 def _check_comp_ch(cch, kind, desired=None):
-    if 'reference' in kind.lower():
-        if cch['grad_order_no'] != 0:
-            raise RuntimeError('%s channel with non-zero compensation grade %s'
-                               % (kind, cch['grad_order_no']))
-    else:
-        if desired is None:
-            desired = cch['grad_order_no']
-        if cch['grad_order_no'] != desired:
-            raise RuntimeError('%s channel with inconsistent compensation '
-                               'grade %s, should be %s'
-                               % (kind, cch['grad_order_no'], desired))
+    if desired is None:
+        desired = cch['grad_order_no']
+    if cch['grad_order_no'] != desired:
+        raise RuntimeError('%s channel with inconsistent compensation '
+                           'grade %s, should be %s'
+                           % (kind, cch['grad_order_no'], desired))
     return desired
 
 
@@ -218,17 +213,14 @@ def _convert_channel_info(res4, t, use_eeg_pos):
             # Set the coil type
             if cch['sensor_type_index'] == CTF.CTFV_REF_MAG_CH:
                 ch['kind'] = FIFF.FIFFV_REF_MEG_CH
-                _check_comp_ch(cch, 'Reference magnetometer')
                 ch['coil_type'] = FIFF.FIFFV_COIL_CTF_REF_MAG
                 nref += 1
                 ch['logno'] = nref
             elif cch['sensor_type_index'] == CTF.CTFV_REF_GRAD_CH:
                 ch['kind'] = FIFF.FIFFV_REF_MEG_CH
                 if off_diag:
-                    _check_comp_ch(cch, 'Reference off-diagonal gradiometer')
                     ch['coil_type'] = FIFF.FIFFV_COIL_CTF_OFFDIAG_REF_GRAD
                 else:
-                    _check_comp_ch(cch, 'Reference gradiometer')
                     ch['coil_type'] = FIFF.FIFFV_COIL_CTF_REF_GRAD
                 nref += 1
                 ch['logno'] = nref
@@ -239,7 +231,8 @@ def _convert_channel_info(res4, t, use_eeg_pos):
                 nmeg += 1
                 ch['logno'] = nmeg
             # Encode the software gradiometer order
-            ch['coil_type'] = ch['coil_type'] | (cch['grad_order_no'] << 16)
+            ch['coil_type'] = int(
+                ch['coil_type'] | (cch['grad_order_no'] << 16))
             ch['coord_frame'] = FIFF.FIFFV_COORD_DEVICE
         elif cch['sensor_type_index'] == CTF.CTFV_EEG_CH:
             coord_frame = FIFF.FIFFV_COORD_HEAD
@@ -252,7 +245,7 @@ def _convert_channel_info(res4, t, use_eeg_pos):
                         warn('EEG electrode (%s) location omitted because of '
                              'missing HPI information' % ch['ch_name'])
                         ch['loc'].fill(np.nan)
-                        coord_frame = FIFF.FIFFV_COORD_CTF_HEAD
+                        coord_frame = FIFF.FIFFV_MNE_COORD_CTF_HEAD
                     else:
                         ch['loc'][:3] = apply_trans(
                             t['t_ctf_head_head'], ch['loc'][:3])
@@ -450,6 +443,7 @@ def _compose_meas_info(res4, coils, trans, eeg):
         eeg = _pick_eeg_pos(info)
     _add_eeg_pos(eeg, trans, info)
     logger.info('    Measurement info composed.')
+    info._unlocked = False
     info._update_redundant()
     return info
 
@@ -478,8 +472,8 @@ def _annotate_bad_segments(directory, start_time, meas_date):
         for f in fid.readlines():
             tmp = f.strip().split()
             desc.append('bad_%s' % tmp[0])
-            onsets.append(np.float(tmp[1]) - start_time)
-            durations.append(np.float(tmp[2]) - np.float(tmp[1]))
+            onsets.append(np.float64(tmp[1]) - start_time)
+            durations.append(np.float64(tmp[2]) - np.float64(tmp[1]))
     # return None if there are no bad segments
     if len(onsets) == 0:
         return None
