@@ -2,7 +2,7 @@
 #          Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #          Teon Brooks <teon.brooks@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import numpy as np
 
@@ -14,6 +14,7 @@ from .pick import pick_types, pick_channels, pick_channels_forward
 from .base import BaseRaw
 from ..evoked import Evoked
 from ..epochs import BaseEpochs
+from ..fixes import pinv
 from ..utils import (logger, warn, verbose, _validate_type, _check_preload,
                      _check_option, fill_doc)
 from ..defaults import DEFAULTS
@@ -52,7 +53,8 @@ def _check_before_reference(inst, ref_from, ref_to, ch_type):
     _check_preload(inst, "Applying a reference")
 
     ch_type = _get_ch_type(inst, ch_type)
-    ch_dict = {ch_type: True, 'meg': False, 'ref_meg': False}
+    ch_dict = {**{type_: True for type_ in ch_type},
+               'meg': False, 'ref_meg': False}
     eeg_idx = pick_types(inst.info, **ch_dict)
 
     if ref_to is None:
@@ -99,7 +101,8 @@ def _check_before_reference(inst, ref_from, ref_to, ch_type):
     # info that a non-CAR has been applied.
     ref_to_channels = pick_channels(inst.ch_names, ref_to, ordered=True)
     if len(np.intersect1d(ref_to_channels, eeg_idx)) > 0:
-        inst.info['custom_ref_applied'] = FIFF.FIFFV_MNE_CUSTOM_REF_ON
+        with inst.info._unlock():
+            inst.info['custom_ref_applied'] = FIFF.FIFFV_MNE_CUSTOM_REF_ON
 
     return ref_to
 
@@ -107,7 +110,6 @@ def _check_before_reference(inst, ref_from, ref_to, ch_type):
 def _apply_reference(inst, ref_from, ref_to=None, forward=None,
                      ch_type='auto'):
     """Apply a custom EEG referencing scheme."""
-    from scipy import linalg
     ref_to = _check_before_reference(inst, ref_from, ref_to, ch_type)
 
     # Compute reference
@@ -135,7 +137,7 @@ def _apply_reference(inst, ref_from, ref_to=None, forward=None,
             # 4. Compute the forward (G) and average-reference it (Ga):
             Ga = G - np.mean(G, axis=0, keepdims=True)
             # 5. Compute the Ga_inv by SVD
-            Ga_inv = linalg.pinv(Ga, rcond=1e-6)
+            Ga_inv = pinv(Ga, rtol=1e-6)
             # 6. Compute Ra = (G @ Ga_inv) in eq (8) from G and Ga_inv
             Ra = G @ Ga_inv
             # 7-8. Compute Vp = Ra @ Va; then Vpa=average(Vp)
@@ -209,9 +211,8 @@ def add_reference_channels(inst, ref_channels, copy=True):
                        dl['kind'] == FIFF.FIFFV_POINT_EEG and
                        dl['ident'] == 0)]
         if len(ref_channels) > 1 or len(ref_dig_loc) != len(ref_channels):
-            ref_dig_array = np.zeros(12)
-            warn('The locations of multiple reference channels are ignored '
-                 '(set to zero).')
+            ref_dig_array = np.full(12, np.nan)
+            warn('The locations of multiple reference channels are ignored.')
         else:  # n_ref_channels == 1 and a single ref digitization exists
             ref_dig_array = np.concatenate((ref_dig_loc[0]['r'],
                                            ref_dig_loc[0]['r'], np.zeros(6)))
@@ -219,12 +220,12 @@ def add_reference_channels(inst, ref_channels, copy=True):
             for idx in pick_types(inst.info, meg=False, eeg=True, exclude=[]):
                 inst.info['chs'][idx]['loc'][3:6] = ref_dig_loc[0]['r']
     else:
-        # we should actually be able to do this from the montage, but
-        # it looks like the montage isn't stored, so we can't extract
-        # this information. The user will just have to call set_montage()
-        # by setting this to zero, we fall back to the old behavior
-        # when missing digitisation
-        ref_dig_array = np.zeros(12)
+        # Ideally we'd fall back on getting the location from a montage, but
+        # locations for non-present channels aren't stored, so location is
+        # unknown. Users can call set_montage() again if needed.
+        ref_dig_array = np.full(12, np.nan)
+        logger.info('Location for this channel is unknown; consider calling '
+                    'set_montage() again if needed.')
 
     for ch in ref_channels:
         chan_info = {'ch_name': ch,
@@ -319,11 +320,14 @@ def set_eeg_reference(inst, ref_channels='average', copy=True,
             # sure that the custom_ref_applied flag is left untouched.
             custom_ref_applied = inst.info['custom_ref_applied']
             try:
-                inst.info['custom_ref_applied'] = FIFF.FIFFV_MNE_CUSTOM_REF_OFF
+                with inst.info._unlock():
+                    inst.info['custom_ref_applied'] = \
+                        FIFF.FIFFV_MNE_CUSTOM_REF_OFF
                 inst.add_proj(make_eeg_average_ref_proj(inst.info,
-                              activate=False))
+                                                        activate=False))
             except Exception:
-                inst.info['custom_ref_applied'] = custom_ref_applied
+                with inst.info._unlock():
+                    inst.info['custom_ref_applied'] = custom_ref_applied
                 raise
             # If the data has been preloaded, projections will no
             # longer be automatically applied.
@@ -336,9 +340,9 @@ def set_eeg_reference(inst, ref_channels='average', copy=True,
 
     inst = inst.copy() if copy else inst
     ch_type = _get_ch_type(inst, ch_type)
-    ch_dict = {ch_type: True, 'meg': False, 'ref_meg': False}
-    eeg_idx = pick_types(inst.info, **ch_dict)
-    ch_sel = [inst.ch_names[i] for i in eeg_idx]
+    ch_dict = {**{type_: True for type_ in ch_type},
+               'meg': False, 'ref_meg': False}
+    ch_sel = [inst.ch_names[i] for i in pick_types(inst.info, **ch_dict)]
 
     if ref_channels == 'REST':
         _validate_type(forward, Forward, 'forward when ref_channels="REST"')
@@ -352,22 +356,34 @@ def set_eeg_reference(inst, ref_channels='average', copy=True,
     if ref_channels == []:
         logger.info('EEG data marked as already having the desired reference.')
     else:
-        logger.info('Applying a custom %s '
-                    'reference.' % DEFAULTS['titles'][ch_type])
+        logger.info(
+            'Applying a custom '
+            f"{tuple(DEFAULTS['titles'][type_] for type_ in ch_type)} "
+            'reference.')
 
     return _apply_reference(inst, ref_channels, ch_sel, forward,
                             ch_type=ch_type)
 
 
 def _get_ch_type(inst, ch_type):
-    _validate_type(ch_type, str, 'ch_type')
-    _check_option('ch_type', ch_type, ('auto', 'eeg', 'ecog', 'seeg', 'dbs'))
+    _validate_type(ch_type, (str, list, tuple), 'ch_type')
+    valid_ch_types = ('auto', 'eeg', 'ecog', 'seeg', 'dbs')
+    if isinstance(ch_type, str):
+        _check_option('ch_type', ch_type, valid_ch_types)
+        if ch_type != 'auto':
+            ch_type = [ch_type]
+    elif isinstance(ch_type, (list, tuple)):
+        for type_ in ch_type:
+            _validate_type(type_, str, 'ch_type')
+            _check_option('ch_type', type_, valid_ch_types[1:])
+        ch_type = list(ch_type)
+
     # if ch_type is 'auto', search through list to find first reasonable
     # reference-able channel type.
     if ch_type == 'auto':
         for type_ in ['eeg', 'ecog', 'seeg', 'dbs']:
             if type_ in inst:
-                ch_type = type_
+                ch_type = [type_]
                 logger.info('%s channel type selected for '
                             're-referencing' % DEFAULTS['titles'][type_])
                 break
@@ -512,7 +528,8 @@ def set_bipolar_reference(inst, anode, cathode, ch_name=None, ch_info=None,
     # Set other info-keys from original instance.
     pick_info = {k: v for k, v in inst.info.items() if k not in
                  ['chs', 'ch_names', 'bads', 'nchan', 'sfreq']}
-    ref_info.update(pick_info)
+    with ref_info._unlock():
+        ref_info.update(pick_info)
 
     # Rereferencing of data.
     ref_data = multiplier @ inst._data
