@@ -1,9 +1,9 @@
 # Authors: Marijn van Vliet <w.m.vanvliet@gmail.com>
 #
-# License: BSD 3 clause
+# License: BSD-3-Clause
 
 from copy import deepcopy
-from os import remove, makedirs
+from os import makedirs
 import os.path as op
 import re
 from shutil import copy
@@ -11,20 +11,19 @@ from shutil import copy
 import numpy as np
 import pytest
 from numpy.testing import assert_equal, assert_allclose
-import matplotlib.pyplot as plt
 
 from mne import (make_bem_model, read_bem_surfaces, write_bem_surfaces,
                  make_bem_solution, read_bem_solution, write_bem_solution,
-                 make_sphere_model, Transform, Info, write_surface)
+                 make_sphere_model, Transform, Info, write_surface,
+                 write_head_bem)
 from mne.preprocessing.maxfilter import fit_sphere_to_headshape
 from mne.io.constants import FIFF
 from mne.transforms import translation
 from mne.datasets import testing
-from mne.utils import (run_tests_if_main, catch_logging,
-                       requires_freesurfer, requires_nibabel)
+from mne.utils import catch_logging, requires_h5py
 from mne.bem import (_ico_downsample, _get_ico_map, _order_surfaces,
                      _assert_complete_surface, _assert_inside,
-                     _check_surface_size, _bem_find_surface, make_flash_bem)
+                     _check_surface_size, _bem_find_surface)
 from mne.surface import read_surface
 from mne.io import read_info
 
@@ -39,6 +38,8 @@ fname_bem_sol_3 = op.join(subjects_dir, 'sample', 'bem',
                           'sample-320-320-320-bem-sol.fif')
 fname_bem_sol_1 = op.join(subjects_dir, 'sample', 'bem',
                           'sample-320-bem-sol.fif')
+fname_dense_head = op.join(subjects_dir, 'sample', 'bem',
+                           'sample-head-dense.fif')
 
 
 def _compare_bem_surfaces(surfs_1, surfs_2):
@@ -68,26 +69,41 @@ def _compare_bem_solutions(sol_a, sol_b):
 
 
 @testing.requires_testing_data
-def test_io_bem(tmpdir):
+@requires_h5py
+@pytest.mark.parametrize('ext', ('fif', 'h5'))
+def test_io_bem(tmp_path, ext):
     """Test reading and writing of bem surfaces and solutions."""
-    temp_bem = op.join(str(tmpdir), 'temp-bem.fif')
-    pytest.raises(ValueError, read_bem_surfaces, fname_raw)
-    pytest.raises(ValueError, read_bem_surfaces, fname_bem_3, s_id=10)
+    import h5py
+    temp_bem = op.join(str(tmp_path), f'temp-bem.{ext}')
+    # model
+    with pytest.raises(ValueError, match='BEM data not found'):
+        read_bem_surfaces(fname_raw)
+    with pytest.raises(ValueError, match='surface with id 10'):
+        read_bem_surfaces(fname_bem_3, s_id=10)
     surf = read_bem_surfaces(fname_bem_3, patch_stats=True)
     surf = read_bem_surfaces(fname_bem_3, patch_stats=False)
     write_bem_surfaces(temp_bem, surf[0])
+    with pytest.raises(IOError, match='exists'):
+        write_bem_surfaces(temp_bem, surf[0])
+    write_bem_surfaces(temp_bem, surf[0], overwrite=True)
+    if ext == 'h5':
+        with h5py.File(temp_bem, 'r'):  # make sure it's valid
+            pass
     surf_read = read_bem_surfaces(temp_bem, patch_stats=False)
     _compare_bem_surfaces(surf, surf_read)
 
-    pytest.raises(RuntimeError, read_bem_solution, fname_bem_3)
-    temp_sol = op.join(str(tmpdir), 'temp-sol.fif')
+    # solution
+    with pytest.raises(RuntimeError, match='No BEM solution found'):
+        read_bem_solution(fname_bem_3)
+    temp_sol = op.join(str(tmp_path), f'temp-sol.{ext}')
     sol = read_bem_solution(fname_bem_sol_3)
     assert 'BEM' in repr(sol)
     write_bem_solution(temp_sol, sol)
     sol_read = read_bem_solution(temp_sol)
     _compare_bem_solutions(sol, sol_read)
     sol = read_bem_solution(fname_bem_sol_1)
-    pytest.raises(RuntimeError, _bem_find_surface, sol, 3)
+    with pytest.raises(RuntimeError, match='BEM does not have.*triangulation'):
+        _bem_find_surface(sol, 3)
 
 
 def test_make_sphere_model():
@@ -122,12 +138,12 @@ def test_make_sphere_model():
 
 @testing.requires_testing_data
 @pytest.mark.parametrize('kwargs, fname', [
-    [dict(), fname_bem_3],
+    pytest.param(dict(), fname_bem_3, marks=pytest.mark.slowtest),  # Azure
     [dict(conductivity=[0.3]), fname_bem_1],
 ])
-def test_make_bem_model(tmpdir, kwargs, fname):
+def test_make_bem_model(tmp_path, kwargs, fname):
     """Test BEM model creation from Python with I/O."""
-    fname_temp = tmpdir.join('temp-bem.fif')
+    fname_temp = tmp_path / 'temp-bem.fif'
     with catch_logging() as log:
         model = make_bem_model('sample', ico=2, subjects_dir=subjects_dir,
                                verbose=True, **kwargs)
@@ -135,8 +151,8 @@ def test_make_bem_model(tmpdir, kwargs, fname):
     if len(kwargs.get('conductivity', (0, 0, 0))) == 1:
         assert 'distance' not in log
     else:
-        assert re.search(r'urfaces is approximately *3\.6 mm', log) is not None
-    assert re.search(r'inner skull CM is *0\.69 *-10\.00 *44\.26 mm',
+        assert re.search(r'urfaces is approximately *3\.4 mm', log) is not None
+    assert re.search(r'inner skull CM is *0\.65 *-9\.62 *43\.85 mm',
                      log) is not None
     model_c = read_bem_surfaces(fname)
     _compare_bem_surfaces(model, model_c)
@@ -150,25 +166,25 @@ def test_make_bem_model(tmpdir, kwargs, fname):
 
 
 @testing.requires_testing_data
-def test_bem_model_topology(tmpdir):
+def test_bem_model_topology(tmp_path):
     """Test BEM model topological checks."""
     # bad topology (not enough neighboring tris)
-    makedirs(tmpdir.join('foo', 'bem'))
+    makedirs(tmp_path / 'foo' / 'bem')
     for fname in ('inner_skull', 'outer_skull', 'outer_skin'):
         fname += '.surf'
         copy(op.join(subjects_dir, 'sample', 'bem', fname),
-             str(tmpdir.join('foo', 'bem', fname)))
-    outer_fname = tmpdir.join('foo', 'bem', 'outer_skull.surf')
+             tmp_path / 'foo' / 'bem' / fname)
+    outer_fname = tmp_path / 'foo' / 'bem' / 'outer_skull.surf'
     rr, tris = read_surface(outer_fname)
     tris = tris[:-1]
     write_surface(outer_fname, rr, tris[:-1], overwrite=True)
     with pytest.raises(RuntimeError, match='Surface outer skull is not compl'):
-        make_bem_model('foo', None, subjects_dir=tmpdir)
+        make_bem_model('foo', None, subjects_dir=tmp_path)
     # Now get past this error to reach gh-6127 (not enough neighbor tris)
     rr_bad = np.concatenate([rr, np.mean(rr, axis=0, keepdims=True)], axis=0)
     write_surface(outer_fname, rr_bad, tris, overwrite=True)
-    with pytest.raises(RuntimeError, match='Surface outer skull.*triangles'):
-        make_bem_model('foo', None, subjects_dir=tmpdir)
+    with pytest.raises(ValueError, match='Surface outer skull.*triangles'):
+        make_bem_model('foo', None, subjects_dir=tmp_path)
 
 
 @pytest.mark.slowtest
@@ -177,7 +193,7 @@ def test_bem_model_topology(tmpdir):
     [(0.3,), fname_bem_sol_1],
     [(0.3, 0.006, 0.3), fname_bem_sol_3],
 ])
-def test_bem_solution(tmpdir, cond, fname):
+def test_bem_solution(tmp_path, cond, fname):
     """Test making a BEM solution from Python with I/O."""
     # test degenerate conditions
     surf = read_bem_surfaces(fname_bem_1)[0]
@@ -202,7 +218,7 @@ def test_bem_solution(tmpdir, cond, fname):
     pytest.raises(RuntimeError, _check_surface_size, surfs[1])
 
     # actually test functionality
-    fname_temp = op.join(str(tmpdir), 'temp-bem-sol.fif')
+    fname_temp = op.join(str(tmp_path), 'temp-bem-sol.fif')
     # use a model and solution made in Python
     for model_type in ('python', 'c'):
         if model_type == 'python':
@@ -374,41 +390,27 @@ def test_fit_sphere_to_headshape():
     pytest.raises(TypeError, fit_sphere_to_headshape, 1, units='m')
 
 
-@requires_nibabel()
-@requires_freesurfer('mri_convert')
+@pytest.mark.slowtest  # ~2 min on Azure Windows
 @testing.requires_testing_data
-def test_make_flash_bem(tmpdir):
-    """Test computing bem from flash images."""
-    tmp = str(tmpdir)
-    bemdir = op.join(subjects_dir, 'sample', 'bem')
-    flash_path = op.join(subjects_dir, 'sample', 'mri', 'flash')
+def test_io_head_bem(tmp_path):
+    """Test reading and writing of defective head surfaces."""
+    head = read_bem_surfaces(fname_dense_head)[0]
+    fname_defect = op.join(str(tmp_path), 'temp-head-defect.fif')
+    # create defects
+    head['rr'][0] = np.array([-0.01487014, -0.04563854, -0.12660208])
+    head['tris'][0] = np.array([21919, 21918, 21907])
 
-    for surf in ('inner_skull', 'outer_skull', 'outer_skin'):
-        copy(op.join(bemdir, surf + '.surf'), tmp)
-        copy(op.join(bemdir, surf + '.tri'), tmp)
-    copy(op.join(bemdir, 'inner_skull_tmp.tri'), tmp)
-    copy(op.join(bemdir, 'outer_skin_from_testing.surf'), tmp)
+    with pytest.raises(ValueError, match='topological defects:'):
+        write_head_bem(fname_defect, head['rr'], head['tris'])
+    with pytest.warns(RuntimeWarning, match='topological defects:'):
+        write_head_bem(fname_defect, head['rr'], head['tris'],
+                       on_defects='warn')
+    # test on_defects in read_bem_surfaces
+    with pytest.raises(ValueError, match='topological defects:'):
+        read_bem_surfaces(fname_defect)
+    with pytest.warns(RuntimeWarning, match='topological defects:'):
+        head_defect = read_bem_surfaces(fname_defect, on_defects='warn')[0]
 
-    # This function deletes the tri files at the end.
-    try:
-        make_flash_bem('sample', overwrite=True, subjects_dir=subjects_dir,
-                       flash_path=flash_path)
-        for surf in ('inner_skull', 'outer_skull', 'outer_skin'):
-            coords, faces = read_surface(op.join(bemdir, surf + '.surf'))
-            surf = 'outer_skin_from_testing' if surf == 'outer_skin' else surf
-            coords_c, faces_c = read_surface(op.join(tmp, surf + '.surf'))
-            assert_equal(0, faces.min())
-            assert_equal(coords.shape[0], faces.max() + 1)
-            assert_allclose(coords, coords_c)
-            assert_allclose(faces, faces_c)
-    finally:
-        for surf in ('inner_skull', 'outer_skull', 'outer_skin'):
-            remove(op.join(bemdir, surf + '.surf'))  # delete symlinks
-            copy(op.join(tmp, surf + '.tri'), bemdir)  # return deleted tri
-            copy(op.join(tmp, surf + '.surf'), bemdir)  # return moved surf
-        copy(op.join(tmp, 'inner_skull_tmp.tri'), bemdir)
-        copy(op.join(tmp, 'outer_skin_from_testing.surf'), bemdir)
-    plt.close('all')
-
-
-run_tests_if_main()
+    assert head['id'] == head_defect['id'] == FIFF.FIFFV_BEM_SURF_ID_HEAD
+    assert np.allclose(head['rr'], head_defect['rr'])
+    assert np.allclose(head['tris'], head_defect['tris'])
