@@ -1,6 +1,6 @@
 # Author: Eric Larson <larson.eric.d@gmail.com>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 import sys
 from mne.utils import run_subprocess
@@ -26,7 +26,7 @@ if len(bad) > 0:
 # check sklearn and others
 for x in sys.modules.keys():
     for key in ('sklearn', 'pandas', 'mayavi', 'pyvista', 'matplotlib',
-                'dipy', 'nibabel', 'cupy', 'picard', 'pyvistaqt'):
+                'dipy', 'nibabel', 'cupy', 'picard', 'pyvistaqt', 'pooch'):
         if x.startswith(key):
             x = '.'.join(x.split('.')[:2])
             out |= {x}
