@@ -8,7 +8,7 @@ from itertools import product
 
 import mne
 from mne.channels import equalize_channels
-from mne.utils import sum_squared, run_tests_if_main, _TempDir, requires_h5py
+from mne.utils import sum_squared, requires_h5py
 from mne.time_frequency import (csd_fourier, csd_multitaper,
                                 csd_morlet, csd_array_fourier,
                                 csd_array_multitaper, csd_array_morlet,
@@ -58,19 +58,19 @@ def test_csd():
 def test_csd_repr():
     """Test string representation of CrossSpectralDensity."""
     csd = _make_csd()
-    assert str(csd) == ('<CrossSpectralDensity  |  n_channels=3, time=0.0 to '
+    assert str(csd) == ('<CrossSpectralDensity | n_channels=3, time=0.0 to '
                         '1.0 s, frequencies=1.0, 2.0, 3.0, 4.0 Hz.>')
 
-    assert str(csd.mean()) == ('<CrossSpectralDensity  |  n_channels=3, '
+    assert str(csd.mean()) == ('<CrossSpectralDensity | n_channels=3, '
                                'time=0.0 to 1.0 s, frequencies=1.0-4.0 Hz.>')
 
     csd_binned = csd.mean(fmin=[1, 3], fmax=[2, 4])
-    assert str(csd_binned) == ('<CrossSpectralDensity  |  n_channels=3, '
+    assert str(csd_binned) == ('<CrossSpectralDensity | n_channels=3, '
                                'time=0.0 to 1.0 s, frequencies=1.0-2.0, '
                                '3.0-4.0 Hz.>')
 
     csd_binned = csd.mean(fmin=[1, 2], fmax=[1, 4])
-    assert str(csd_binned) == ('<CrossSpectralDensity  |  n_channels=3, '
+    assert str(csd_binned) == ('<CrossSpectralDensity | n_channels=3, '
                                'time=0.0 to 1.0 s, frequencies=1.0, 2.0-4.0 '
                                'Hz.>')
 
@@ -78,7 +78,7 @@ def test_csd_repr():
     csd_no_time.tmin = None
     csd_no_time.tmax = None
     assert str(csd_no_time) == (
-        '<CrossSpectralDensity  |  n_channels=3, time=unknown, '
+        '<CrossSpectralDensity | n_channels=3, time=unknown, '
         'frequencies=1.0, 2.0, 3.0, 4.0 Hz.>'
     )
 
@@ -221,10 +221,10 @@ def test_csd_get_data():
 
 
 @requires_h5py
-def test_csd_save():
+def test_csd_save(tmp_path):
     """Test saving and loading a CrossSpectralDensity."""
     csd = _make_csd()
-    tempdir = _TempDir()
+    tempdir = str(tmp_path)
     fname = op.join(tempdir, 'csd.h5')
     csd.save(fname)
     csd2 = read_csd(fname)
@@ -236,10 +236,10 @@ def test_csd_save():
     assert csd._is_sum == csd2._is_sum
 
 
-def test_csd_pickle():
+def test_csd_pickle(tmp_path):
     """Test pickling and unpickling a CrossSpectralDensity."""
     csd = _make_csd()
-    tempdir = _TempDir()
+    tempdir = str(tmp_path)
     fname = op.join(tempdir, 'csd.dat')
     with open(fname, 'wb') as f:
         pickle.dump(csd, f)
@@ -536,7 +536,8 @@ def test_csd_morlet():
     # Test baselining warning
     epochs_nobase = epochs.copy()
     epochs_nobase.baseline = None
-    epochs_nobase.info['highpass'] = 0
+    with epochs_nobase.info._unlock():
+        epochs_nobase.info['highpass'] = 0
     with pytest.warns(RuntimeWarning, match='baseline'):
         csd = csd_morlet(epochs_nobase, frequencies=[10], decim=20)
 
@@ -549,6 +550,3 @@ def test_equalize_channels():
 
     assert csd1.ch_names == ['CH1', 'CH2']
     assert csd2.ch_names == ['CH1', 'CH2']
-
-
-run_tests_if_main()
