@@ -2,7 +2,7 @@
 """Testing functions."""
 # Authors: Alexandre Gramfort <alexandre.gramfort@inria.fr>
 #
-# License: BSD (3-clause)
+# License: BSD-3-Clause
 
 from contextlib import contextmanager
 from distutils.version import LooseVersion
@@ -19,18 +19,11 @@ import warnings
 
 import numpy as np
 from numpy.testing import assert_array_equal, assert_allclose
-from scipy import linalg
 
-from ._logging import warn
+from ._logging import warn, ClosingStringIO
 from .numerics import object_diff
 
 
-def nottest(f):
-    """Mark a function as not a test (decorator)."""
-    f.__test__ = False
-    return f
-
-
 def _explain_exception(start=-1, stop=None, prefix='> '):
     """Explain an exception."""
     # start=-1 means "only the most recent caller"
@@ -73,7 +66,7 @@ def requires_nibabel():
 def requires_dipy():
     """Check for dipy."""
     import pytest
-    # for some strange reason on CIs we cane get:
+    # for some strange reason on CIs we can get:
     #
     #     can get weird ImportError: dlopen: cannot load any more object
     #     with static TLS
@@ -234,13 +227,16 @@ def _import_mlab():
 @contextmanager
 def traits_test_context():
     """Context to raise errors in trait handlers."""
-    from traits.api import push_exception_handler
-
-    push_exception_handler(reraise_exceptions=True)
     try:
+        from traits.api import push_exception_handler
+    except Exception:
         yield
-    finally:
-        push_exception_handler(reraise_exceptions=False)
+    else:
+        push_exception_handler(reraise_exceptions=True)
+        try:
+            yield
+        finally:
+            push_exception_handler(reraise_exceptions=False)
 
 
 def traits_test(test_func):
@@ -252,18 +248,6 @@ def traits_test(test_func):
     return dec
 
 
-@nottest
-def run_tests_if_main():
-    """Run tests in a given file if it is run as a script."""
-    local_vars = inspect.currentframe().f_back.f_locals
-    if local_vars.get('__name__', '') != '__main__':
-        return
-    import pytest
-    code = pytest.main([local_vars['__file__'], '-v'])
-    if code:
-        raise AssertionError('pytest finished with errors (%d)' % (code,))
-
-
 def run_command_if_main():
     """Run a given command if it's __main__."""
     local_vars = inspect.currentframe().f_back.f_locals
@@ -277,8 +261,8 @@ class ArgvSetter(object):
     def __init__(self, args=(), disable_stdout=True,
                  disable_stderr=True):  # noqa: D102
         self.argv = list(('python',) + args)
-        self.stdout = StringIO() if disable_stdout else sys.stdout
-        self.stderr = StringIO() if disable_stderr else sys.stderr
+        self.stdout = ClosingStringIO() if disable_stdout else sys.stdout
+        self.stderr = ClosingStringIO() if disable_stderr else sys.stderr
 
     def __enter__(self):  # noqa: D105
         self.orig_argv = sys.argv
@@ -298,12 +282,17 @@ class ArgvSetter(object):
 class SilenceStdout(object):
     """Silence stdout."""
 
+    def __init__(self, close=True):
+        self.close = close
+
     def __enter__(self):  # noqa: D105
         self.stdout = sys.stdout
         sys.stdout = StringIO()
         return sys.stdout
 
     def __exit__(self, *args):  # noqa: D105
+        if self.close:
+            sys.stdout.close()
         sys.stdout = self.stdout
 
 
@@ -373,8 +362,8 @@ def _raw_annot(meas_date, orig_time):
     raw = RawArray(data=np.empty((10, 10)), info=info, first_samp=10)
     if meas_date is not None:
         meas_date = _handle_meas_date(meas_date)
-    raw.info['meas_date'] = meas_date
-    raw.info._check_consistency()
+    with raw.info._unlock(check_after=True):
+        raw.info['meas_date'] = meas_date
     annot = Annotations([.5], [.2], ['dummy'], orig_time)
     raw.set_annotations(annotations=annot)
     return raw
@@ -444,6 +433,7 @@ def assert_meg_snr(actual, desired, min_tol, med_tol=500., chpi_med_tol=500.,
 
 def assert_snr(actual, desired, tol):
     """Assert actual and desired arrays are within some SNR tolerance."""
+    from scipy import linalg
     with np.errstate(divide='ignore'):  # allow infinite
         snr = (linalg.norm(desired, ord='fro') /
                linalg.norm(desired - actual, ord='fro'))
@@ -469,17 +459,31 @@ def assert_dig_allclose(info_py, info_bin, limit=None):
     """Assert dig allclose."""
     from ..bem import fit_sphere_to_headshape
     from ..io.constants import FIFF
+    from ..io.meas_info import Info
+    from ..channels.montage import DigMontage
     # test dig positions
-    dig_py = sorted(info_py['dig'], key=_dig_sort_key)
-    dig_bin = sorted(info_bin['dig'], key=_dig_sort_key)
+    dig_py, dig_bin = info_py, info_bin
+    if isinstance(dig_py, Info):
+        assert isinstance(dig_bin, Info)
+        dig_py, dig_bin = dig_py['dig'], dig_bin['dig']
+    else:
+        assert isinstance(dig_bin, DigMontage)
+        assert isinstance(dig_py, DigMontage)
+        dig_py, dig_bin = dig_py.dig, dig_bin.dig
+        info_py = info_bin = None
+    assert isinstance(dig_py, list)
+    assert isinstance(dig_bin, list)
+    dig_py = sorted(dig_py, key=_dig_sort_key)
+    dig_bin = sorted(dig_bin, key=_dig_sort_key)
     assert len(dig_py) == len(dig_bin)
     for ii, (d_py, d_bin) in enumerate(zip(dig_py[:limit], dig_bin[:limit])):
         for key in ('ident', 'kind', 'coord_frame'):
-            assert d_py[key] == d_bin[key]
+            assert d_py[key] == d_bin[key], key
         assert_allclose(d_py['r'], d_bin['r'], rtol=1e-5, atol=1e-5,
                         err_msg='Failure on %s:\n%s\n%s'
                         % (ii, d_py['r'], d_bin['r']))
-    if any(d['kind'] == FIFF.FIFFV_POINT_EXTRA for d in dig_py):
+    if any(d['kind'] == FIFF.FIFFV_POINT_EXTRA for d in dig_py) and \
+            info_py is not None:
         r_bin, o_head_bin, o_dev_bin = fit_sphere_to_headshape(
             info_bin, units='m', verbose='error')
         r_py, o_head_py, o_dev_py = fit_sphere_to_headshape(
@@ -514,3 +518,15 @@ def modified_env(**d):
                 os.environ[key] = val
             elif key in os.environ:
                 del os.environ[key]
+
+
+def _click_ch_name(fig, ch_index=0, button=1):
+    """Click on a channel name in a raw/epochs/ICA browse-style plot."""
+    from ..viz.utils import _fake_click
+    fig.canvas.draw()
+    text = fig.mne.ax_main.get_yticklabels()[ch_index]
+    bbox = text.get_window_extent()
+    x = bbox.intervalx.mean()
+    y = bbox.intervaly.mean()
+    _fake_click(fig, fig.mne.ax_main, (x, y), xform='pix',
+                button=button)
